/******************************************************************************
 * File Name          : MeasureCurrent.ino
 * Description        : Readme file
 * Date               : 2019-04-08
 ******************************************************************************
 *
 * Copyright (c) 2019 Waveshare
 * All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS, IMPLIED OR STATUTORY WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
 * PARTICULAR PURPOSE AND NON-INFRINGEMENT OF THIRD PARTY INTELLECTUAL PROPERTY
 * RIGHTS ARE DISCLAIMED TO THE FULLEST EXTENT PERMITTED BY LAW. IN NO EVENT 
 * SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************/

//this is the logical:
//when press,then check that Raspberry Pi'Power,if power off,then power on.
//when release,then check chat Raspberry Pi'Power,if power on，then power off(soft shutdown or cut the power).
 
 
#include "PowerManagementHAT.h"
#include "Time.h"
#include "LowPower.h"
#include "PCF8523.h"
#include <Wire.h>

//-------------------------------------------------------------------------------------

//when key pressed more than SHUTDOWN_TIME_MS,start soft shudown
#define SHUTDOWN_TIME_MS    2000

//when key pressed more than CUT_POWER_TIME_MS,just cut the power
#define CUT_POWER_TIME_MS   8000

//-------------------------------------------------------------------------------------

//define whether button pressed value
volatile bool  buttonPressed = false;

//time for RTC to wake pi up periodically when buttonState in buttonState_Wait state
eTIMER_TIMEBASE  Timer_Timebase     = eTB_SECOND;
uint8_t          Timer_Value        = 5;


//button press interrupt
void button_isr();
//alarm press interrupt
void alarm_isr();



void setup()
{
  
  //SetButtoninterrupt 
  power.SetButtoninterrupt(&buttonPressed,button_isr);  
  //Init power
  power.Init();

}






void loop() 
{


	float piRunCurrent;

	//sleep and set alarm time  ：if any interrupt wil stop sleep and run
	power.Sleep(Timer_Timebase,Timer_Value,alarm_isr);
	
	//Use button for startup and shutdown functions
	power.ButtonRun(SHUTDOWN_TIME_MS,CUT_POWER_TIME_MS);

	piRunCurrent = power.measureCurrent();
	Serial.print("The Raspberry Pi running current: ");
	Serial.print(piRunCurrent);  
	Serial.println(" mA");  

}








//button press interrupt
void button_isr()
{
	
    buttonPressed = true; //it's necessary
	//Here you can add some functions when buttonPressed
	
}

//alarm press interrupt
void alarm_isr()
{
	
	//Here you can add some functions when alarm ，Trigger every five seconds by default
		
}

