/******************************************************************************
 * File Name          : MeasureVoltage_ShutdownInLowVoltage.ino
 * Description        : Readme file
 * Date               : 2019-04-08
 ******************************************************************************
 *
 * Copyright (c) 2019 Waveshare
 * All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS, IMPLIED OR STATUTORY WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
 * PARTICULAR PURPOSE AND NON-INFRINGEMENT OF THIRD PARTY INTELLECTUAL PROPERTY
 * RIGHTS ARE DISCLAIMED TO THE FULLEST EXTENT PERMITTED BY LAW. IN NO EVENT 
 * SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************/

#include "PowerManagementHAT.h"
#include "Time.h"
#include "LowPower.h"
#include "PCF8523.h"
#include <Wire.h>


//-------------------------------------------------------------------------------------

#define SHUTDOWN_TIME_MS    2000
#define CUT_POWER_TIME_MS   8000

//                          	1S(not support)    2s    3s  	 4S
//ON_VOLTAGE_VOLT                   (3.3)         6.6  	9.9	 	13.2
//OFF_VOLTAGE_VOLT                  (3.15)        6.3   9.45	12.6
//FORCE_OFF_VOLTAGE_VOLT            (2.9)         5.8   8.7 	11.6

//
#define	ON_VOLTAGE_VOLT          6.6

//
#define OFF_VOLTAGE_VOLT         6.3

//
#define FORCE_OFF_VOLTAGE_VOLT   5.8

//very low voltage more than this time will start a shudown
#define LOW_VOLTAGE_TIME_MS   	(30000ul)

//if botton pressed,then you will have more time	1h/5s=720
#define OVERRIDE_TIME_MS      	(3600000ul)

//-------------------------------------------------------------------------------------

//define whether button pressed value
volatile bool  buttonPressed = false;

//define whether alarm Fired value
volatile bool  alarmFired = false;

//time for RTC to wake pi up periodically when buttonState in buttonState_Wait state
eTIMER_TIMEBASE  Timer_Timebase     = eTB_SECOND;
uint8_t          Timer_Value        = 5;

//The maximum count defaults to increments every 5 seconds
//Used to calculate low voltage time    
unsigned long 	 maxcount			= LOW_VOLTAGE_TIME_MS /(Timer_Value*1000);

//button press interrupt
void button_isr();
//alarm press interrupt
void alarm_isr();
//button press 0-2s
void button_even();


void setup()
{
  //Set the voltage to power on or off
  power.SetOnVoltage(ON_VOLTAGE_VOLT);
  power.SetOffVoltage(OFF_VOLTAGE_VOLT);
  power.SetForceOffVoltage(FORCE_OFF_VOLTAGE_VOLT);
  
  //SetButtoninterrupt 
  power.SetButtoninterrupt(&buttonPressed,button_isr);  
  
  power.SetAlarminterrupt(&alarmFired,alarm_isr);
  //Init power
  power.Init();
  
}






void loop() 
{

	float supply_voltage;
	

	//sleep and set alarm time  ：if any interrupt wil stop sleep and run
	power.Sleep(Timer_Timebase,Timer_Value);
	
	//Use button for startup and shutdown functions
	power.ButtonRun(SHUTDOWN_TIME_MS,CUT_POWER_TIME_MS,button_even);
	
	//get supply_voltage and print
	supply_voltage = power.measureVoltage();
	Serial.println("supply_voltage:"+(String)supply_voltage);
	
	//LowVoltageCheck 
	power.LowVoltageCheck(maxcount);
	
	//Automatic startup over ON_VOLTAGE_VOLT
	//power.VoltageBoot(ON_VOLTAGE_VOLT);
	power.VoltageBoot();

}
//when buttonPressed  less 2s
void button_even()
{
	
	maxcount=maxcount+(OVERRIDE_TIME_MS/(Timer_Value*1000));
	//Here you can add some functions when buttonPressed  less 2s
	
}






//button press interrupt
void button_isr()
{
	
    buttonPressed = true; //it's necessary
	//Here you can add some functions when buttonPressed
	
}

//alarm press interrupt
void alarm_isr()
{
	alarmFired = true;	//it's necessary
	//Here you can add some functions when alarm ，Trigger every five seconds by default
		
}
