/******************************************************************************
 * File Name          : PowerOnInAPeriodTime.ino
 * Description        : Readme file
 * Date               : 2019-04-08
 ******************************************************************************
 *
 * Copyright (c) 2019 Waveshare
 * All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS, IMPLIED OR STATUTORY WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
 * PARTICULAR PURPOSE AND NON-INFRINGEMENT OF THIRD PARTY INTELLECTUAL PROPERTY
 * RIGHTS ARE DISCLAIMED TO THE FULLEST EXTENT PERMITTED BY LAW. IN NO EVENT 
 * SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************/

#include "PowerManagementHAT.h"
#include "time_info.h"
#include "LowPower.h"
#include "PCF8523.h"
#include <Wire.h>
#include "Time.h"



const int USER_LED = 13;


//-------------------------------------------------------------------------------------

//Wakeup Hour in 24 hour clock
#define  WAKE_UP_START_HOUR      8
//Wakeup Minute
#define  WAKE_UP_START_MINUTE    30

//Sleep Houe in 24 hour clock
#define  SLEEP_START_HOUR       18
//Sleep Minute
#define  SLEEP_START_MINUTE     30 
//default,your pi will wakeUp in 8:30，Sleep in 18:30

//-------------------------------------------------------------------------------------


bool  piAwake = false;

void alarm_isr()
{
	//RTC clock alarm interrupt
}

void setup()
{

	
	//initialize 
	power.Init();
	
	Serial.println("System is woking now");
	delay(60);


	// Just set the time(Compiled) to RTC clock.
	SetTime();

	//print the time in serial monitor
	printTimeNow();

	Serial.print("Alarm Set for: ");
	Serial.print(WAKE_UP_START_HOUR);
	Serial.print(":");
	Serial.println(WAKE_UP_START_MINUTE);

	Serial.print("Bedtime Set for: ");
	Serial.print(SLEEP_START_HOUR);
	Serial.print(":");
	Serial.println(SLEEP_START_MINUTE);
	delay(100);

}

void loop() 
{
	if(piAwake == false)
	{
		//RegularSleep by RTC alarm  	default wake up at 8:30
		power.RegularSleep(WAKE_UP_START_HOUR,WAKE_UP_START_MINUTE);
			
		//NOW,the alarm clock rings.
		//in here,you can do some task
		digitalWrite(USER_LED,LOW);
		Serial.println("wake up");
		
		//Print the time
		printTimeNow();
		
		//Print the time requires time
		delay(50);
		
		//turn off the LED
		digitalWrite(USER_LED,HIGH);
		
		//Connect the power to wakeup the Pi
		power.connectPiPower(true); 
		piAwake = true;      
	}
	else
	{	
		//RegularSleep by RTC alarm  	default sleep at 18:30
		power.RegularSleep(SLEEP_START_HOUR,SLEEP_START_MINUTE);	
		
		//NOW,the alarm clock rings.
		//in here,you can do some task
		digitalWrite(USER_LED,LOW);
		Serial.println("go to bed now!");

		//Print the time
		printTimeNow();

		//print the time requires time
		delay(50);

		digitalWrite(USER_LED,HIGH);

		//Start a soft shutdown to the pi
		power.ShutdownPi();         
		piAwake = false;       
	}
}

