/******************************************************************************
 * File Name          : PowerOnPeriodicallyOnAlarm.ino
 * Description        : Readme file
 * Date               : 2019-04-08
 ******************************************************************************
 *
 * Copyright (c) 2019 Waveshare
 * All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS, IMPLIED OR STATUTORY WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
 * PARTICULAR PURPOSE AND NON-INFRINGEMENT OF THIRD PARTY INTELLECTUAL PROPERTY
 * RIGHTS ARE DISCLAIMED TO THE FULLEST EXTENT PERMITTED BY LAW. IN NO EVENT 
 * SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************/

#include "PowerManagementHAT.h"
#include "time_info.h"
#include "LowPower.h"
#include "PCF8523.h"
#include "Time.h"
#include <Wire.h>

const int USER_LED = 13;

//-------------------------------------------------------------------------------------

//Raspberry Pi awake time in minute 
#define  RPI_WAKE_UP_CYCLE_MINUTE   10

//Raspberry Pi Stay Awake time in ms
#define  RPI_STAY_AWAKE_TIME_MS     60000

//When then current is below this, It is considered to be shutdown
#define  CURRENT_THRESHOLD_MA       110

//-------------------------------------------------------------------------------------



void alarm_isr()
{
	//alarm interrupt
}

void setup()
{
	//initialize power 
	power.SetCurrentThreshold(CURRENT_THRESHOLD_MA);
	power.Init();
	Serial.println("I am going to wake up periodically by alarm clock");
	delay(100);

	power.connectPiPower(false);

	//Just set the time(Compiled) to RTC clock.
	SetTime();
	
	//print the time in serial monitor
	printTimeNow();

	Serial.print("Raspberry Pi Wakeup Cycle is:");
	Serial.print(RPI_WAKE_UP_CYCLE_MINUTE);     
	Serial.println(" minutes");         

	//Calculate the next time to power on
	nextWakeTime = CalcNextWakeTime(RPI_WAKE_UP_CYCLE_MINUTE);
}


void loop() 
{  
	//RegularSleep by RTC alarm  	default 10 minutes
    power.RegularSleep(nextWakeTime);
	
    //Calculate the next time to power pi
    nextWakeTime = CalcNextWakeTime(RPI_WAKE_UP_CYCLE_MINUTE);

	//Power pi on
    power.connectPiPower(true);    

	//Light the LED
    digitalWrite(USER_LED,LOW);
	
    Serial.println("I've Just woken up on the Alarm!");
	
    //Print the time
    printTimeNow();
	
	//print the time requires time,so delay
    delay(50);
	
	//turn off the LED
    digitalWrite(USER_LED,HIGH); 

	//you can do some other task here
	
	//wait to boot up a bit,of course,this is included in RPI_STAY_AWAKE_TIME_MS
    delay(10000);
	
    power.DelayShutdown(RPI_STAY_AWAKE_TIME_MS);


}


