/******************************************************************************
 * File Name          : PowerOnPeriodicallyOnTimer.ino
 * Description        : Readme file
 * Date               : 2019-04-08
 ******************************************************************************
 *
 * Copyright (c) 2019 Waveshare
 * All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS, IMPLIED OR STATUTORY WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
 * PARTICULAR PURPOSE AND NON-INFRINGEMENT OF THIRD PARTY INTELLECTUAL PROPERTY
 * RIGHTS ARE DISCLAIMED TO THE FULLEST EXTENT PERMITTED BY LAW. IN NO EVENT 
 * SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************/

#include "PowerManagementHAT.h"
#include "Time.h"
#include "LowPower.h"
#include "PCF8523.h"
#include <Wire.h>
#include "time_info.h"


const int USER_LED = 13;

//-------------------------------------------------------------------------------------

//Timebase set to seconds. Other options: eTB_SECOND, eTB_MINUTE, eTB_HOUR
//eTIMER_TIMEBASE  PERIODIC_TIMER_TIMEBASE = eTB_MINUTE;

#define  PERIODIC_TIMER_TIMEBASE     eTB_MINUTE

//The timer interval in unit of PERIODIC_TIMER_TIMEBASE
#define  PERIODIC_TIMER_VALUE        10

//RPI stay awake time
#define  RPI_STAY_AWAKE_TIME_MS      60000

//-------------------------------------------------------------------------------------
void Setting_info(void);
void alarm_isr(void);

void setup()
{ 

	//initialize
	power.Init();
	Serial.println("Starting, but I'm going to go to sleep for a while...");
	delay(60);
	

	//Just set the time(Compiled) to RTC clock.
	SetTime();

	//print the time in serial monitor
	printTimeNow();   

	//print Setting information
	Setting_info();
}

void loop() 
{
	//sleep default 5 minutes
    power.Sleep(PERIODIC_TIMER_TIMEBASE,PERIODIC_TIMER_VALUE,alarm_isr);

    //Then wake the Pi
    power.connectPiPower(true);

	//Turn on the LED
    digitalWrite(USER_LED,LOW);
	
    Serial.println("I am waking!!!");
	
    //Print the time
    printTimeNow();
	
	//print the time requires time
    delay(50);
	
    digitalWrite(USER_LED,HIGH);

	//This is the stay awake time
    delay(RPI_STAY_AWAKE_TIME_MS);        

    //Has been awaked for a time,then shutdown.
    power.shutPiDown();
     
}

void  Setting_info(void)
{
	Serial.print("Periodic Interval Set for: ");
	Serial.print(PERIODIC_TIMER_VALUE);
	switch(PERIODIC_TIMER_TIMEBASE)
	{
	case eTB_SECOND:
	  Serial.print(" seconds");
	  break;
	case eTB_MINUTE:
	  Serial.print(" minutes");
	  break;
	case eTB_HOUR:
	  Serial.print(" hours");
	default:
		Serial.print(" unknown timebase");
		break;
	}
	Serial.print(" + ");
	Serial.print(RPI_STAY_AWAKE_TIME_MS / 1000);
	Serial.println(" seconds");	
	delay(10);
}
void alarm_isr(void)
{
	//The handle of RTC alarm
}
