/******************************************************************************
 * File Name          : PowerOnPeriodicallyOnTimerInAPeriodTime.ino
 * Description        : Readme file
 * Date               : 2019-04-08
 ******************************************************************************
 *
 * Copyright (c) 2019 Waveshare
 * All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS, IMPLIED OR STATUTORY WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
 * PARTICULAR PURPOSE AND NON-INFRINGEMENT OF THIRD PARTY INTELLECTUAL PROPERTY
 * RIGHTS ARE DISCLAIMED TO THE FULLEST EXTENT PERMITTED BY LAW. IN NO EVENT 
 * SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************/

#include "PowerManagementHAT.h"
#include "Time.h"
#include "LowPower.h"
#include "PCF8523.h"
#include <Wire.h>
#include "time_info.h"


const int LED_PIN = 13;

//-------------------------------------------------------------------------------------

//Timebase set to seconds. Other options: eTB_SECOND, eTB_MINUTE, eTB_HOUR
//eTIMER_TIMEBASE  PERIODIC_TIMER_TIMEBASE = eTB_MINUTE;

#define  PERIODIC_TIMER_TIMEBASE  eTB_MINUTE

//Raspberry Pi awake time in minute 
#define  PERIODIC_TIMER_VALUE     5

//Raspberry Pi Stay Awake time in ms
#define  RPI_STAY_AWAKE_TIME_MS   60000

//Wakeup Hour in 24 hour clock
#define  WAKE_UP_START_HOUR       8
//Wakeup Minute
#define  WAKE_UP_START_MINUTE     30
//Sleep Houe in 24 hour clock
#define  SLEEP_START_HOUR        18
//Sleep Minute
#define  SLEEP_START_MINUTE      30

//-------------------------------------------------------------------------------------


void  Setting_info(void);
void alarm_isr(void);


void setup()
{ 
	// initialize serial communication
	power.Init();
	Serial.println("I am goning to WakePiPeriodicallyInSpecifiedTime");
	delay(60);

	//Just set the time(Compiled) to RTC clock.
	SetTime();
	
	//print the time in serial monitor
	printTimeNow();   
	
	//print Setting information
	Setting_info();

}

void loop() 
{
    DateTime now;
	//sleep default 5 minutes
    power.Sleep(PERIODIC_TIMER_TIMEBASE,PERIODIC_TIMER_VALUE,alarm_isr);

	//NOW,the alarm clock rings.
	//in here,you can do some task
    digitalWrite(LED_PIN,LOW);
    Serial.println("I wake up periodically during the specified time");
	
    //Print the time
    printTimeNow();
	
	//print the time requires time
    delay(50);
	
	//turn off LED
    digitalWrite(LED_PIN,HIGH);

	//read RTC time
    now = power.readTime();
		
	//check whether the time is in specified time
	if((now.hour()*60 + now.minute()) > (WAKE_UP_START_HOUR*60 + WAKE_UP_START_MINUTE)){
		if((now.hour()*60 + now.minute()) < (SLEEP_START_HOUR*60 + SLEEP_START_MINUTE)){

			//if it is,then power on the pi
			power.connectPiPower(true);
			
			//the pi is powered on,and delay for RPI_STAY_AWAKE_TIME_MS ms
			delay(RPI_STAY_AWAKE_TIME_MS);    
			
			//has already wake for that period,so shutdown
			power.shutPiDown();
		}
	}
	
}






void  Setting_info(void)
{
	Serial.print("Periodic Interval Set for: ");
	Serial.print(PERIODIC_TIMER_VALUE);
	switch(PERIODIC_TIMER_TIMEBASE)
	{
	case eTB_SECOND:
	  Serial.print(" seconds");
	  break;
	case eTB_MINUTE:
	  Serial.print(" minutes");
	  break;
	case eTB_HOUR:
	  Serial.print(" hours");
	default:
		Serial.print(" unknown timebase");
		break;
	}
	Serial.print(" + ");
	Serial.print(RPI_STAY_AWAKE_TIME_MS / 1000);
	Serial.println(" seconds");
}



void alarm_isr(void)
{
	//RTC clock alarm interrupt
}

