/******************************************************************************
 * File Name          : Power-Management-HAT.cpp
 * Description        : Readme file
 * Date               : 2019-04-08
 ******************************************************************************
 *
 * Copyright (c) 2019 Waveshare
 * All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS, IMPLIED OR STATUTORY WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
 * PARTICULAR PURPOSE AND NON-INFRINGEMENT OF THIRD PARTY INTELLECTUAL PROPERTY
 * RIGHTS ARE DISCLAIMED TO THE FULLEST EXTENT PERMITTED BY LAW. IN NO EVENT 
 * SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************/
#include "PowerManagementHAT.h"

#define ARD_ENABLE_PI		16				// PC 2|______|O/P|______|take high to enable the RaspPi-------------------______|Active High
#define ARD_PI_SHUT_DOWN	17				// PC 3|______|0/P|______|Handshake to request the Pi to shutdown----------______|Active high
#define ARD_PI_IS_RUNNING	7				// PD 7|______|I/P|______|Handshake to show that the Pi is running---------______|Active High  
#define ARD_USER_BUTTON		3				// PD 3|______|I/P|______|User Power-on Button (INT1)----------------------______|Active Low
#define ARD_RTC_ALARM		2				// PD 2|______|I/P|______|Pin that pulses when the alarm has expired (INT0)______|Active Low
#define USER_LED 			13				// PD13|______|O/P|______|Driver LED pin-----------------------------------______|LED PIN
#define VOLTAGE				A6				//  I/P|______|A/I|______|Supply monitoring pin----------------------------______|ADC Pin
#define CURRENT				A7				//  I/P|______|A/I|______|Current monitoring pin---------------------------______|ADC Pin	

bool default_button =false;
bool default_Alarm	=false;

void nothing(void)
{
	;
}
void default_Buttoninterrupt(void)
{

	default_button=true;

}
void default_Alarminterrupt(void)
{

	default_Alarm=true;
	//power.ackTimer1();

}
void default_error(void)
{
	char count;
	digitalWrite(USER_LED,HIGH);
	delay(1000);
	for(count=0;count<3;count++)
	{
		digitalWrite(USER_LED,LOW);
		delay(1000);
		digitalWrite(USER_LED,HIGH);
		delay(1000);
	}
}
//Constructor
PowerManagement::PowerManagement(){
	
	//----------------some value-------------------------
		
	//pi statue
	isPiRunning = false;
	
	//when ARD_ENABLE_PI become high level,then powwerOn become true
	powerOn = false;

	//when use current to check the status of pi(warning:default use pin to check pi status)
	USE_CURRENT=false;
	
	//default turn off the current
	Current_Threshold_Ma =85;

	//default Buttoninterrupt funtion
	Buttoninterrupt=default_Buttoninterrupt;

	//default Alarminterrupt funtion
	Alarminterrupt=default_Alarminterrupt;

	//default Buttonflag
	Buttonflag=&default_button;

	//default Alarmflag
	Alarmflag=&default_Alarm;

	//default deal boot error funtion : LED Three flashes Duration 1s
	BootError=default_error;

	//default MAX boot time :60S
	MAxBootTime=60000;

	error_flag=false;

	ON_VOLTAGE=6.6;

	OFF_VOLTAGE=6.3;

	FORCE_OFF_VOLTAGE=5.8;

	//----------------output pin-------------------------
	
	//Configure Pi Power
	pinMode(ARD_ENABLE_PI, OUTPUT);
	
	//Pi is Off at Startup
	PowerManagement::connectPiPower(false);
	
	//configure ARD_PI_SHUT_DOWN pin to output,pull up will shutdown pi
	pinMode(ARD_PI_SHUT_DOWN, OUTPUT);
	
	//at the start up pull down mean don't shutdown pi
	digitalWrite(ARD_PI_SHUT_DOWN,LOW);	
	
	//----------------input pin-------------------------
	
	//Configure Pi Shutdown handshake from Pi goes high when Pi is running
	pinMode(ARD_PI_IS_RUNNING, INPUT);    
	
	//Initialize the button pin as a input,also can be used as an interrupt INT1
	pinMode(ARD_USER_BUTTON, INPUT);
	
	//Initialize the alarm as a input,also can be used as an interrupt INT0
	pinMode(ARD_RTC_ALARM, INPUT);	

	//LDE PIN ,Control LED on and off by high and low level
	pinMode(USER_LED, OUTPUT);

	//turn off LED
	digitalWrite(USER_LED,HIGH);
}

/* 
 * brief : enable normal power
 * param : true   if enable normal power
 *		   false  disable normal power
 */
void PowerManagement::connectPiPower(bool enable){
	if(enable == true){
		digitalWrite(ARD_ENABLE_PI,HIGH);
		powerOn = true;
	}
	else {
		digitalWrite(ARD_ENABLE_PI,LOW);
		powerOn = false;
	}
}

/*
 *brief:
 *		shutDown the Raspberry Pi by pull ARD_PI_SHUT_DOWN up
 *		wait ARD_PI_IS_RUNNING become low level means Pi has shudown
 *		when ARD_PI_IS_RUNNING become low level or timeout,then cut the power
 */
void PowerManagement::shutPiDown(void){
	int	handShake;
	unsigned long timeStart, timeNow, testTime;

	//pull up to send shutdown commander
	digitalWrite(ARD_PI_SHUT_DOWN,HIGH);		
	
	//wait for max 60s for Pi to shutdown 
	timeStart = millis();
	testTime = 0;
	handShake = digitalRead(ARD_PI_IS_RUNNING);
	while((handShake > 0) && (testTime < MAX_SHUT_DOWN_TIME)){
		handShake = digitalRead(ARD_PI_IS_RUNNING);  
		delay(50);
		timeNow = millis();
		testTime = timeNow - timeStart;
	}
	//when program comes to here,means has wait for max 60s(MAX_SHUT_DOWN_TIME)
	delay(6000);
	//so directly to cut the power
	PowerManagement::connectPiPower(false);
	digitalWrite(ARD_PI_SHUT_DOWN,LOW);
}

/*
 *brief:
 *		shutDown the Raspberry Pi by pull ARD_PI_SHUT_DOWN up
 *		wait currtnt become below thresholdCurrent_mA
 *		when current become below thresholdCurrent_mA or timeout,then cut the power
 */
void PowerManagement::shutPiDown(long thresholdCurrent_mA){
	bool isPiRunning;
	unsigned long timeStart, timeNow, testTime;
	
	// Command the Sleepy Pi to shutdown
	digitalWrite(ARD_PI_SHUT_DOWN,HIGH);		
		
	// Wait for max 60s for Pi to shutdown
	timeStart = millis();
	testTime = 0;
	isPiRunning = PowerManagement::getPiStatus(thresholdCurrent_mA,false);
	while((isPiRunning == true) && (testTime < MAX_SHUT_DOWN_TIME)){
		isPiRunning = PowerManagement::getPiStatus(thresholdCurrent_mA,false); 
		delay(50);
		timeNow = millis();
		testTime = timeNow - timeStart;
	}
	//when program comes to here,means has wait for max 60s(MAX_SHUT_DOWN_TIME)
	delay(6000);
	//so directly cut the power
	PowerManagement::connectPiPower(false);
	digitalWrite(ARD_PI_SHUT_DOWN,LOW);
}

/* 
 * brief : Check if Pi is running
 * param : forceShutdownIfNotWorking:
 * if pi is not running,then shutdown 	   
 */
bool PowerManagement::getPiStatus(bool forceShutdownIfNotWorking){
	
	int	handShake;
	
	handShake = digitalRead(ARD_PI_IS_RUNNING);	
	
	//if handShake>0,means pi is still running
	if(handShake > 0) {
		isPiRunning = true;
		return true;
	}else{
		//need to force shutdown?
		if(forceShutdownIfNotWorking == true){
			//it was running
			if(isPiRunning == true){
				//but now,isn't running,cut the power directly
				PowerManagement::connectPiPower(false);
				isPiRunning = false;
			}
		}
		return false;
	}
}

/* 
 * brief : check Pi status by current
 * param : 
 *		   thresholdCurrent_mA:
 *		   if working current is more than thresholdCurrent_mA, then it is working
 *
 *         forceShutdownIfNotWorking:
 *         need to shutdown ,if it is not running
 */
bool PowerManagement::getPiStatus(long thresholdCurrent_mA, bool forceShutdownIfNotWorking){

	float electricCurrent = 0.0;

	electricCurrent = PowerManagement::measureCurrent();
	if(electricCurrent >= (float)thresholdCurrent_mA){
		isPiRunning = true;
		return true;
	}else{
		if(forceShutdownIfNotWorking == true){
			PowerManagement::connectPiPower(false);
			isPiRunning = false;
		}
		return false;
	}
}

/*
 * brief: read the voltage to predict the power voltage
 */
float PowerManagement::measureVoltage(void){
	int 	tempVoltage;
	float   powerVoltage;
	
	tempVoltage = analogRead(VOLTAGE);   
	
	// 3.3V/1024 = 3.22mV
	powerVoltage = 3.22*(float)tempVoltage;
	// covert it to reallt voltage
	powerVoltage /= 52;
	
	return powerVoltage;
}

/*
 * brief: read the voltage to predict the power voltage
 */
float PowerManagement::measureCurrent(void){
	int 	tempCurrent;
	float   powerCurrent;

	tempCurrent = analogRead(CURRENT);  
	//filter the noise
	if(tempCurrent <= 3){
		tempCurrent = 0;
	}
	
	//3.3V/1024 = 3.22mV
	powerCurrent = 3.22*(float)tempCurrent;
	
	return  powerCurrent;           
}

/*
 *brief: Init the RTC clock
 *params: reset
 *		true: reset the RTC clock
 *		false: does't reset the RTC clock
 */
bool PowerManagement::RTCInit(bool reset){

	uint8_t tmp_reg;
	begin();
	
	//do this to check whether PCF8523 is exist
	tmp_reg = rtcReadReg(PCF8523_CONTROL_3);
	if(tmp_reg == 0xFF){
		//means PCF8523 is not detected
		return false;
	}

	//then reset the clock now
	if(reset){
		PowerManagement::reset();
	}
	
	//do some deal with it RTC clock
	PowerManagement::stop_32768_clkout();  
	PowerManagement::setBatterySwitchover();
	PowerManagement::rtcCapSelect(eCAP_12_5pF);
	PowerManagement::clearRtcInterruptFlags();

	return true;
}

/* 
 * brief : enable RTC alarm
 * param : true   if enable RTC alarm
 *		   false  disable RTC alarm
 * This interface is used for ino files
 */
void PowerManagement::RTCAlarmClockEnable(bool enable){
	enableAlarm(enable);
}

/*
 * brief: clear RTC interrupt
 * Thia interface is used for ino files
 */
uint8_t PowerManagement::RTCClearInterrupts(){	
	return clearRtcInterruptFlags();
}




/*New funtion by eng33*/



/*
 * brief: set sleep time and sleep
 * param :
 * 		Timer_Timebase	:	Unit of sleep time
 * 		Timer_Value		:	Value of sleep time
 * 		interrupt		:	Function Pointers are used to pass alarm interrupt functions
 */
void PowerManagement::Sleep(eTIMER_TIMEBASE Timer_Timebase,uint8_t Timer_Value,void (* interrupt)(void)){	
	
	
	PowerManagement::RTCClearInterrupts();
	  
	attachInterrupt(0, interrupt, FALLING);

	PowerManagement::setTimer1(Timer_Timebase, Timer_Value);

	//arduino goes to low power model
	PowerManagement::powerDown(SLEEP_FOREVER, ADC_OFF, BOD_OFF); 


}
/*
 * brief: set sleep time and sleep
 * param :
 * 		Timer_Timebase	:	Unit of sleep time
 * 		Timer_Value		:	Value of sleep time
 * 		interrupt		:	Function Pointers are used to pass alarm interrupt functions
 */
void PowerManagement::Sleep(eTIMER_TIMEBASE Timer_Timebase,uint8_t Timer_Value){	

	PowerManagement::Sleep(Timer_Timebase,Timer_Value,Alarminterrupt);
}
/*
 * brief: set check pi way
 * param :
 * 		CheckWay: if true  ,CheckPiStatus and ShutdownPi will use cureent to cheak pi	
 */
void PowerManagement::SetCheckWay(bool CheckWay){	
	USE_CURRENT=CheckWay;
}

/*
 * brief: set Current Threshold
 * param :
 * 		CurrentThreshold: 	Raspberry PI shutdown current
 */
void PowerManagement::SetCurrentThreshold(long CurrentThreshold){	
	Current_Threshold_Ma=CurrentThreshold;
}

/*
 * brief: Check Raspberry Pi tatus
 * param :
 * 		forceShutdownIfNotWorking: 	need to shutdown ,if it is not running
 */
bool PowerManagement::CheckPiStatus(bool forceShutdownIfNotWorking){	
	
	if(USE_CURRENT==true) return PowerManagement::getPiStatus(Current_Threshold_Ma,forceShutdownIfNotWorking);
	else return PowerManagement::getPiStatus(forceShutdownIfNotWorking);

}

/*
 * brief: Shutdown Raspberry Pi
 */
void PowerManagement::ShutdownPi(void){	
	if(USE_CURRENT==true)PowerManagement::shutPiDown(Current_Threshold_Ma);
	else PowerManagement::shutPiDown();
}

/*
 * brief: Set Button interrupt
 * param :
 * 		flag	: 	The pointer to the button that presses the flag
 * 	Interrupt	:	Function Pointers are used to pass Button interrupt functions
 * 		
 */
void PowerManagement::SetButtoninterrupt(bool *flag,void(*Interrupt)(void)){
	Buttonflag=flag;
	Buttoninterrupt=Interrupt;
}

/*
 * brief: Set alarm interrupt
 * param :
 * 	Interrupt	:	Function Pointers are used to pass alarm interrupt functions
 * 		
 */
void PowerManagement::SetAlarminterrupt(bool *flag,void(*Interrupt)(void)){
	Alarmflag=flag;
	Alarminterrupt=Interrupt;
}



/*
 * brief		: Handle the various cases when a button is pressed
 * param :
 * 	SHUTDOWN_TIME	: 	If the button is pressed for longer than that but less than CUT_POWER_TIME,will be try shutdown Raspberry Pi
 * 	CUT_POWER_TIME	:	If the button is pressed longer than that, the power will be cut off
 * 	NotEnough		:	If the button is pressed less than SHUTDOWN_TIME ,will be do something	
 */
void PowerManagement::ButtonRun(unsigned long SHUTDOWN_TIME,unsigned long CUT_POWER_TIME,void (*NotEnough)(void)){

	unsigned long time=0, timePress=0;
	int buttonPressedTime;
	
	digitalWrite(USER_LED,LOW);
	if(*Buttonflag==true)
	{
		detachInterrupt(1);
		detachInterrupt(0);
		isPiRunning=PowerManagement::CheckPiStatus(false);	
		if((isPiRunning==true)||(error_flag==true))
		{
			delay(10);
			attachInterrupt(1, Buttoninterrupt, HIGH);
			*Buttonflag=false;
			timePress = millis();	
			while(*Buttonflag==false){
				
				delay(5);
			}
			detachInterrupt(1);
			time=millis();
			buttonPressedTime=time-timePress;
			Serial.print("buttonPressedTime:");
			Serial.println(buttonPressedTime);
			if(buttonPressedTime>CUT_POWER_TIME)
			{
				PowerManagement::connectPiPower(false);
				error_flag=false;
			}
			else if(buttonPressedTime>SHUTDOWN_TIME)
			{
				PowerManagement::ShutdownPi();
				error_flag=false;
			}
			else
			{
				NotEnough();
			}			
		}
		else
		{
			PowerManagement::connectPiPower(true);
			time = millis();		
			do{
				delay(100);
				if((millis()-time)>MAxBootTime)
				{
					BootError();
					error_flag=true;
					break;
				}
			}while(PowerManagement::CheckPiStatus(false)==false);
			if(USE_CURRENT==true)delay(10000);
			//Serial.println("OPEN");	
		}	
		*Buttonflag=false;	
	}
	delay(5);	
	attachInterrupt(1, Buttoninterrupt, LOW);
	digitalWrite(USER_LED,HIGH);
	//PowerManagement::ackTimer1(); 
}	

/*
 * brief		: Handle the various cases when a button is pressed
 * param :
 * 	SHUTDOWN_TIME	: 	If the button is pressed for longer than that but less than CUT_POWER_TIME,will be try shutdown Raspberry Pi
 * 	CUT_POWER_TIME	:	If the button is pressed longer than that, the power will be cut off
 */
void PowerManagement::ButtonRun(unsigned long SHUTDOWN_TIME,unsigned long CUT_POWER_TIME)
{
	PowerManagement::ButtonRun(SHUTDOWN_TIME,CUT_POWER_TIME,nothing);
}

/*
 * brief			: Set the boot error handler
 * param :
 * 		Error		: A pointer to the boot error handler
 */
void PowerManagement::SetBootError(void (*Error)(void))
{
	BootError=Error;
}



/*
 * brief			: Set the maximum normal boot time
 * param :
 * 		time		: the maximum normal boot time
 */
void PowerManagement::SetMaxBootTime(unsigned long time)
{
	MAxBootTime=time;
}

/*
 * brief			: Set default ON_VOLTAGE
 * param :
 * 		Voltage		: Boot voltage
 */
void PowerManagement::SetOnVoltage(float Voltage)
{
	ON_VOLTAGE=Voltage;
}


/*
 * brief			: Set default OFF_VOLTAGE
 * param :
 * 		Voltage		: shutdown voltage
 */
void PowerManagement::SetOffVoltage(float Voltage)
{
	OFF_VOLTAGE=Voltage;
}


/*
 * brief			: Set default OFF_VOLTAGE
 * param :
 * 		Voltage		: cut off power voltage
 */
void PowerManagement::SetForceOffVoltage(float Voltage)
{
	FORCE_OFF_VOLTAGE=Voltage;
}


/*
 * brief					: Set default VOLTAGE
 * param :
 * 		ON_Voltage			: 	boot voltage
 * 		OFF_Voltage			:	shutdown voltage
 * 		FORCE_OFF_Voltage	:	cut off power voltage
 */
void PowerManagement::SetVoltage(float ON_Voltage,float OFF_Voltage,float FORCE_OFF_Voltage)
{
	ON_VOLTAGE			=	ON_Voltage;
	OFF_VOLTAGE			=	OFF_Voltage;
	FORCE_OFF_VOLTAGE	=	FORCE_OFF_Voltage;
}


/*
 * brief: Power on when the voltage is higher than input Voltage
 * 	param :
 * 		Voltage		:	Battery voltage above this value will be boot
 */
void PowerManagement::VoltageBoot(float Voltage)
{
	float supply_voltage;
	unsigned long time;
	supply_voltage=PowerManagement::measureVoltage();
	if(supply_voltage>Voltage)
	{
		connectPiPower(true);
		time = millis();
		digitalWrite(USER_LED,LOW);	
		detachInterrupt(1);
		detachInterrupt(0);	
		do{
			delay(100);
			if((millis()-time)>MAxBootTime)
			{
				BootError();
				error_flag=true;
				break;
			}
		}while(PowerManagement::CheckPiStatus(false)==false);
		*Buttonflag=false;
		attachInterrupt(1, Buttoninterrupt, LOW);
		digitalWrite(USER_LED,HIGH);
	}

}


/*
 * brief: Power on when Battery voltage is higher than ON_VOLTAGE
 */
void PowerManagement::VoltageBoot(void)
{
	PowerManagement::VoltageBoot(ON_VOLTAGE);
}

/*
 * brief: Low Voltage Check
 * param :
 * 		cnt		:	Low voltage time count . Shutdown Pi occurs when count is greater than or equal to cnt
 */
void PowerManagement::LowVoltageCheck(unsigned long cnt)
{
	static bool LowVoltageflag =false;
	//static unsigned long begin_time=0,end_time=0;
	static unsigned char count=0;
	float supply_voltage;
	supply_voltage=PowerManagement::measureVoltage();
	isPiRunning=PowerManagement::CheckPiStatus(false);	
	if((isPiRunning==true)||(error_flag==true))
	{
		if(supply_voltage<=FORCE_OFF_VOLTAGE)
		{
			PowerManagement::connectPiPower(false);
			error_flag=false;
		}
		else if(supply_voltage<=OFF_VOLTAGE)
		{
			count++;
			if(count>=cnt&&(PowerManagement::measureVoltage()<=OFF_VOLTAGE))
			{
				digitalWrite(USER_LED,LOW);
				detachInterrupt(1);
				detachInterrupt(0);
				PowerManagement::ShutdownPi();
				*Buttonflag=false;
				attachInterrupt(1, Buttoninterrupt, LOW);				
				digitalWrite(USER_LED,HIGH);
				count=0;
				error_flag=false;
				PowerManagement::Sleep(eTB_SECOND,5);
			}
		}
		else 
		{
			count=0;
		}
	}

}



/*
 * 		brief		: Initialization function
 */
void PowerManagement::Init(void)
{
	power.connectPiPower(false);
	Serial.begin(115200);
	attachInterrupt(1, Buttoninterrupt, LOW);
	PowerManagement::RTCInit(true);

}


/*
 * brief: set sleep time and sleep
 * param :
 * 		WAKE_UP_START_HOUR		:	sleep hour
 * 		WAKE_UP_START_MINUTE	:	sleep minute
 */
void PowerManagement::RegularSleep(uint8_t WAKE_UP_START_HOUR,uint8_t WAKE_UP_START_MINUTE){	

	power.RTCClearInterrupts();

	//allow RTC alarm pin to trigger interrupt
	attachInterrupt(0, Alarminterrupt, FALLING);

	//enable RTC alarm clock
	power.RTCAlarmClockEnable(true);

	//configue the wake up time
	power.setAlarm(WAKE_UP_START_HOUR,WAKE_UP_START_MINUTE);  
	//just like enter the low power mode
	//it will wake up util the wakeup pin is low(just like the alert rings)
	power.powerDown(SLEEP_FOREVER, ADC_OFF, BOD_OFF); 
	//Alarm,and then disable the interrupt
	detachInterrupt(0);

	//ack the alarm clock.
	power.ackAlarm();

}


/*
 * brief: set sleep time and sleep
 * param :
 * 		WAKE_UP_START_MINUTE	:	Sleep time
 */
void PowerManagement::RegularSleep(uint8_t WAKE_UP_START_MINUTE){	

	power.RTCClearInterrupts();

	//allow RTC alarm pin to trigger interrupt
	attachInterrupt(0, Alarminterrupt, FALLING);

	//enable RTC alarm clock
	power.RTCAlarmClockEnable(true);

	//configue the wake up time
	power.setAlarm(WAKE_UP_START_MINUTE);  
	//just like enter the low power mode
	//it will wake up util the wakeup pin is low(just like the alert rings)
	power.powerDown(SLEEP_FOREVER, ADC_OFF, BOD_OFF); 
	//Alarm,and then disable the interrupt
	detachInterrupt(0);

	//ack the alarm clock.
	power.ackAlarm();
}
/*
 * brief: delay shut down
 * param :
 * 		Timer_Timebase	:	Unit of sleep time
 * 		Timer_Value		:	Value of sleep time
 * 		interrupt		:	Function Pointers are used to pass alarm interrupt functions
 */
void PowerManagement::DelayShutdown(unsigned long delaytime){	

	bool pi_running;
	unsigned long TimeOutStart, ElapsedTimeMs,TimeOutEnd ;
	TimeOutStart = millis();
    ElapsedTimeMs = TimeOutStart;
    TimeOutEnd = TimeOutStart + delaytime;
    pi_running = power.getPiStatus(Current_Threshold_Ma,false); 

	//this time,which is pi wakeup time in RPI_STAY_AWAKE_TIME_MS
    while((pi_running == true) && (ElapsedTimeMs < TimeOutEnd))
    {
        pi_running = power.getPiStatus(Current_Threshold_Ma,false); 
        ElapsedTimeMs = millis();
        delay(1000);
		
		//just for debug and tuning to see current change
		Serial.print("state:");
        Serial.print(pi_running);
		Serial.print(" ");
        Serial.print("current:");
        Serial.println(power.measureCurrent());
    }      
    //after run in RPI_STAY_AWAKE_TIME_MS minutes,still running
    if(pi_running == true){
        //then start a shutdown,this will cut the power
        power.shutPiDown();
    }
    else {
        //already shutdown(user shutdown),then just cut the power
        power.connectPiPower(false);   
    }
	
}
/*END*/



PowerManagement power;